c***********************************************************************
      SUBROUTINE enatin(iopc)
c***********************************************************************
C
C   This subroutine calls NWChem for an atomic calculation
C   If one of the reactants or products in an atomic species,
C   it calls this subroutine instead of geoin and secin (since
C   optimization or hessian calculation are meaningless).
C
C   Modified version of Gaussrate version, MAI Nov. 12, 2006
C
      use perconparam
      use common_inc, only : x,dx,iatsv,v
      use nwchemrate_param; use nwchemrate_comm
      IMPLICIT REAL(8) (A-H,O-Z)
      DIMENSION TMPCOORD(N3TM),TMPDX(N3TM),DMFMAT(N3TM,N3TM)
      character*80 syscall
c
c     If it is a Grate restart, read the energy
c
      ircm=1
      irout=1
      iop = iopc
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) then
            ircm=0
            return
      endif
c
      TMPCOORD(:)=0.D0
      TMPDX(:)=0.D0
C
C   Run the NWChem jobs
C
      if(iopc.eq.1)then
         syscall='./taxi esp.fu71 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.2)then
         syscall='./taxi esp.fu72 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.3)then
         syscall='./taxi esp.fu73 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.4)then
         syscall='./taxi esp.fu74 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.5)then
         syscall='./taxi esp.fu75 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.7)then
         syscall='./taxi esp.fu77 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.8)then
         syscall='./taxi esp.fu78 esp.fu82'//procnw
      call system(syscall)
      endif
C
C     if iop=2 or 4, we shift the optimized geometry, so that is does not
C     coincide with the iop=1 geometry
C
      if (iopc.eq.2.or.iopc.eq.4) then
         shft=30.D0
       else
         shft=0.D0
      endif
C
C   And now, we get the information from the NWChem output
C
      NACAT=1
      CALL rdnwout(0,V,TMPCOORD,TMPDX,DMFMAT,IOPC,IATSV,
     *NACAT,NATOMS,NRATOM)
c
c   Now, we need a reordering of the coordinates and gradients
c
      do 1 j=1,natom
         do 2 k=1,natom
            if (j.eq.iatsv(k,iopc)) then
                do 3 L=1,3
                 X((j-1)*3+L)=shft
                 DX((j-1)*3+L)=0.D0
3               continue
            endif
2        continue
1        continue
c
c     If it is a Grate restart, write the energy
c
      if (irest.eq.1) then
          irout=5
          call grest(irout,dummy)
      endif
      return
      end
c
c*****
c
c***********************************************************************
       SUBROUTINE escener(NATESC,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,NATOM,LABEL
     *,IATSV,V,COORD,JTYPE,NACAT,NRATOM)
c***********************************************************************
C
C      Modified version of Gaussrate version, Nov. 10, 2006 by MAI
C
      use perconparam
      use nwchemrate_param; use nwchemrate_comm
      IMPLICIT REAL(8) (A-H,O-Z)
      DIMENSION LABEL(NATESC),COORD(N3TM),NRATOM(8)
      DIMENSION IATSV(NATESC,8)
      DIMENSION DMDX(N3TM),DMX(N3TM),DMFMAT(N3TM,N3TM)
      character*80 syscall
      character*4 scftypej
C
C   First of all, write the NWChem input file
C
C Debug
        if(debug) then
          ntot=nratom(jtype)
          do ii=1,ntot
           write(666,*) 'JTYPE: ',jtype
           write(666,*)'IATSV: ',iatsv(ii,jtype)
          enddo
        endif
      CALL nwinpfl(1,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,LABEL,iatsv,
     *COORD,jtype,nacat,NATESC,nratom)
C
C   Run the NWChem job
C
      syscall='./taxi esp.fu81 esp.fu82'//procnw
      call system(syscall)
C
C   And now, we get the information from G03/G98/G94 output
C
      CALL rdnwout (1,V,DMX,DMDX,DMFMAT,JTYPE,IATSV,
     *nacat,NATESC,NRATOM)
c
      RETURN
C
      END
c
c*****
c
c***********************************************************************
       SUBROUTINE escfirst(NATESC,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,
     *LABEL,IATSV,V,COORD,DX,JTYPE,NACAT,NRATOM)
c***********************************************************************
C
      use perconparam
      use nwchemrate_param; use nwchemrate_comm
      IMPLICIT REAL(8) (A-H,O-Z)
      DIMENSION LABEL(NATESC),COORD(N3TM),DX(N3TM),NRATOM(8)
      DIMENSION IATSV(NATESC,8)
      DIMENSION DMX(N3TM),TMPDX(N3TM),DMFMAT(N3TM,N3TM)
      character*80 syscall
      character*4 scftypej
C
C   First, compare the geometry with the stored one, so we don't have to
C   repeat first derivative calculations (the geometres have to be identical:
C   no rotations or translations are allowed). If they are identical,
C   the first derivative information will be readed instead of calculated
C   and the program will return IDOP=1
C
      IDOP=0
      call store(V,COORD,DX,FMAT,IDOP)
      if (IDOP.EQ.1) then
         return
      endif
C
C   Write Gaussian input options
C
      CALL nwinpfl(2,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,LABEL,iatsv,
     *COORD,jtype,nacat,NATESC,NRATOM)
C
C   Now, run the NWChem job
C
      write(procnw,'(I4)')nwproc
      syscall='./taxi esp.fu81 esp.fu82 '//procnw
      CALL SYSTEM(syscall)
C
C   And now, we get the information from G03/G98/G94 output
C
      CALL rdnwout(2,V,DMX,TMPDX,DMFMAT,JTYPE,IATSV,
     *nacat,NATESC,NRATOM)
c
c     If the system is not the saddle point or any point along the
c     reaction coordinate, reordering of the gradients is required
c
      if (jtype.ne.5.and.jtype.ne.6) then                               1128JC97
      do 1 j=1,natom
         do 2 k=1,natom
            if (j.eq.iatsv(k,jtype)) then
                do 3 L=1,3
                 DX((j-1)*3+L)=TMPDX((k-1)*3+L)
3               continue
            endif
2        continue
1        continue
      else
        do 4 nj=1,natom
             do 5 L=1,3
                 DX((nj-1)*3+L)=TMPDX((nj-1)*3+L)
5       continue
4       continue
      endif
c
      RETURN
      END
c
c*****
c
c***********************************************************************
       SUBROUTINE escsec(NATESC,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,LABEL,
     *IATSV,V,COORD,DX,FMAT,JTYPE,NACAT,NRATOM)
c***********************************************************************
C
      use perconparam
      use nwchemrate_param; use nwchemrate_comm
      IMPLICIT REAL(8) (A-H,O-Z)
      DIMENSION LABEL(NATESC),COORD(N3TM),DX(N3TM),FMAT(N3TM,N3TM)
      DIMENSION IATSV(NATESC,8)                                         1128JC97
      DIMENSION DMX(N3TM),TMPDX(N3TM),NRATOM(8)
      character*80 syscall
      character*4 scftypej
C
C   First of all, write Gaussian input options
C
      CALL nwinpfl(3,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,LABEL,iatsv,
     *COORD,jtype,nacat,NATESC,NRATOM)
C
C   Run the NWChem job
C
      syscall='./taxi esp.fu81 esp.fu82'//procnw
      call system(syscall)
C
C   And now, we get the information from the NWChem output file
C
      CALL rdnwout (3,V,DMX,TMPDX,FMAT,JTYPE,IATSV,
     *nacat,NATESC,NRATOM)
c
c     If the system is not the saddle point or any point along the
c     reaction coordinate, reordering of the gradients is required
c
      if (jtype.ne.5.and.jtype.ne.6) then                               1128JC97
      do 1 j=1,natom
         do 2 k=1,natom
            if (j.eq.iatsv(k,jtype)) then
                do 3 L=1,3
                 DX((j-1)*3+L)=TMPDX((k-1)*3+L)
3               continue
            endif
2        continue
1        continue
      else
        do 4 nj=1,natom
             do 5 L=1,3
                 DX((nj-1)*3+L)=TMPDX((nj-1)*3+L)
5       continue
4       continue
      endif
c
C     We will store V,COORD,DX and FMAT so, in case we need it again
C     for the same geometry, we will have it
C
      CALL store(v,coord,dx,fmat,1)
C
      RETURN
C
      END
c
c*****
c
c***********************************************************************
       SUBROUTINE geoin(iopc,ircm)
c***********************************************************************
c
c     This subroutine calls NWChem for optimizing a stationary point,
c     receiving the optimized geometry, energy of the optimized
c     stationary point and residual gradients after the optimization.
c
      use perconparam
      use common_inc, only : nratom,x,dx,iatsv,v
      use nwchemrate_param; use nwchemrate_comm
      IMPLICIT REAL(8) (A-H,O-Z)
      DIMENSION TMPCOORD(N3TM),TMPDX(N3TM),DUMFMAT(N3TM,N3TM)
      character*80 syscall
c
c     If it is a NWrate restart, read the geom, energ and 1st der.
c
      ircm=1
      irout=4
      if (irest.eq.1) call grest(irout,dummy)
      if (irout.eq.0) then
            ircm=0
            return
      endif
c
c     Inicialize matrices
c
      do 99 ininic=1,n3tm
      TMPCOORD(ininic)=0.D0
      TMPDX(ininic)=0.D0
99    continue
C
C   Run the NWChem jobs
C
      if(iopc.eq.1)then
         syscall='./taxi esp.fu71 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.2)then
         syscall='./taxi esp.fu72 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.3)then
         syscall='./taxi esp.fu73 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.4)then
         syscall='./taxi esp.fu74 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.5)then
         syscall='./taxi esp.fu75 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.7)then
         syscall='./taxi esp.fu77 esp.fu82'//procnw
      call system(syscall)
      endif
      if(iopc.eq.8)then
         syscall='./taxi esp.fu78 esp.fu82'//procnw
      call system(syscall)
      endif
c
c     if iop=2 or 4, we shift the optimized geometry, so that is does not
c     coincide with the iop=1 geometry
c
      if (iopc.eq.2.or.iopc.eq.4) then
         shft=30.D0
       else
         shft=0.D0
      endif
c
c   And now, we get the information from the NWChem output
c
      if (iopc.ne.5) then                                               1128JC97
          NACAT=NRATOM(iopc)
      else
          NACAT=NATOM
      endif
      CALL rdnwout (2,V,TMPCOORD,TMPDX,DUMFMAT,IOPC,IATSV,
     *NACAT,NATOMS,NRATOM)
c
c   Restore the geometry and gradients of the atoms
c   excluded in the optimization
c
      if (iopc.ne.5.and.iopc.ne.6) then                                 1128JC97
c
c   If it is not the saddle point, reordering is needed, since not
c   all the atoms are included in all the reactants and products
c
      do 1 j=1,natom
         do 2 k=1,natom
            if (j.eq.iatsv(k,iopc)) then
                do 3 L=1,3
                 X((j-1)*3+L)=TMPCOORD((k-1)*3+L)+shft
                 DX((j-1)*3+L)=TMPDX((k-1)*3+L)
3               continue
            endif
2        continue
1        continue
      else
c
c   If it is the saddle point, all the atoms are present in it
c
        do 4 nj=1,natom
             do 5 L=1,3
                 X((nj-1)*3+L)=TMPCOORD((nj-1)*3+L)
                 DX((nj-1)*3+L)=TMPDX((nj-1)*3+L)
5       continue
4       continue
      endif
C
c
c     If it is a Grate restart, write the geom., energy and 1st der.
c
      if (irest.eq.1) then
          call rchmul(iopc)
          irout=8
          call grest(irout,dummy)
      endif
      RETURN
      END
c
c*****
c
c***********************************************************************
      SUBROUTINE secin(iopc)
c***********************************************************************
C
C
C   This subroutine calls NWChem for a second derivative calculation.
C
      use perconparam
      use common_inc, only : nratom,x,dx,iatsv,v
      use cm, only : ivrc,label
      use nwchemrate_param; use nwchemrate_comm
      use nwr_eshscm
      IMPLICIT REAL(8) (A-H,O-Z)
C     dimension dmdx(n3tm),dmx(n3tm)                                    0701TA03
C     Variables dmdx and dmx were not used so                           0701TA03
C     there were taken out in Gaussrate 9.1.                            0701TA03
      dimension tmpcoord(n3tm),tmpdx(n3tm)
      character*80 syscall
      character*4 scftypej
C
c     If it is a NWrate restart, read the energy, 1st and 2nd der.
c
      irout=3
      if (irest.eq.1) call grest(irout,fmat)
      if (irout.eq.0) return
c
      if (iopc.ne.5.and.iopc.ne.6) then                                 1128JC97
         NACAT=NRATOM(iopc)
         ichargej=icharge(iopc)
         imultiplj=imultipl(iopc)
         scftypej=scftype(iopc)
      else
         NACAT=NATOM
         ichargej=icharge(5)
         imultiplj=imultipl(5)
         scftypej=scftype(5)
      endif
C
C   The first step is writing the G03/G98/G94 input file
C
      CALL nwinpfl(3,ICHARGEJ,IMULTIPLJ,SCFTYPEJ,LABEL,iatsv,X,
     *iopc,nacat,NATOMS,NRATOM)
C
C   Run the NWChem Job
C
      syscall='./taxi esp.fu81 esp.fu82'//procnw
      call system(syscall)
C
C   And now, we get the information from the NWChem output: Geometry,
C   energy, gradients and hessian.
C
      CALL rdnwout (3,V,TMPCOORD,TMPDX,FMAT,IOPC,IATSV,
     *NACAT,NATOMS,NRATOM)
C
c   Restore the geometry and gradients of the atoms
c   excluded in the optimization
c
      if (iopc.ne.5.and.iopc.ne.6) then                                 1128JC97
c
c   If it is not the saddle point, reordering is needed, since not
c   all the atoms are included in all the reactants and products
c
      do 1 j=1,natom
         do 2 k=1,natom
            if (j.eq.iatsv(k,iopc)) then
                do 3 L=1,3
                 X((j-1)*3+L)=TMPCOORD((k-1)*3+L)
                 DX((j-1)*3+L)=TMPDX((k-1)*3+L)
3               continue
            endif
2        continue
1        continue
      else
c
c   If it is the saddle point, all the atoms are present in it
c
        do 4 nj=1,natom
             do 5 L=1,3
                 X((nj-1)*3+L)=TMPCOORD((nj-1)*3+L)
                 DX((nj-1)*3+L)=TMPDX((nj-1)*3+L)
5       continue
4       continue
      endif
c
c     If it is a NWrate restart, write the energy, 1st and 2nd der.
c
      if (irest.eq.1) then
          irout=7
          call grest(irout,fmat)
      endif
      RETURN
C
      END
c
c*****
c
